// $Id: MotePlugin.java,v 1.13.4.4 2003/09/10 03:41:52 mdwelsh Exp $

/*									tab:2
 *
 *
 * "Copyright (c) 2000 and The Regents of the University 
 * of California.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright
 * notice and the following two paragraphs appear in all copies of
 * this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Authors:	Nelson Lee
 * Date:        December 11 2002
 * Desc:        Default Mote Plugin
 *              Those who want to implement their own MotePlugin should
 *              create a class that extends MotePlugin!
 *
 */

/**
 * @author Nelson Lee
 */


package net.tinyos.sim;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

import net.tinyos.sim.event.*;

public class MotePlugin extends Plugin implements SimConst {
  private static final boolean DEBUG = false;

  public final Color BASIC_COLOR = Color.black;
  public final Color SELECTED_COLOR = new Color(150, 150, 255);
  public final Color BASIC_COLOR_OFF = Color.lightGray;
  public final Color SELECTED_COLOR_OFF = SELECTED_COLOR;
  private static final double GRID_RANDOM_DEVIATION = 5;

  public static final int LAYOUT_RANDOM = 0;
  public static final int LAYOUT_GRID = 1;
  public static final int LAYOUT_GRID_RANDOM = 2;
  public static final int LAYOUT_FILE = 3;
  private int layout = LAYOUT_RANDOM;

  private SimState state;
  private MotePanel motePanel;
  private SimComm simComm;
  private Random rand = new Random();
  private SpatialReader spatialReader;

  // SimObjectGenerators MotePlugin registers
  MoteSimObjectGenerator moteSimObjectGenerator;

  public void initialize(TinyViz tv, JPanel pluginPanel) {
    super.initialize(tv, pluginPanel);
    this.motePanel = tv.getMotePanel();
    this.moteSimObjectGenerator = new MoteSimObjectGenerator(tv);
    this.state = tv.getSimState();
    this.simComm = tv.getSimComm();
  }

  public void register() {
    state.addSOGenerator(moteSimObjectGenerator);
    JButton bPower = tv.getMenuBar().addIconButton("On/off", "ui/power.gif");
    bPower.addActionListener(new bPowerListener());

    // Add layout menu
    ButtonGroup group = new ButtonGroup();
    menuListener ml = new menuListener();
    JMenu layoutMenu = new JMenu("Layout");

    JRadioButtonMenuItem layoutRandom = new JRadioButtonMenuItem("Random");
    group.add(layoutRandom);
    layoutRandom.setFont(tv.defaultFont);
    layoutRandom.addActionListener(ml);
    layoutRandom.setSelected(true);

    JRadioButtonMenuItem layoutGrid = new JRadioButtonMenuItem("Grid");
    group.add(layoutGrid);
    layoutGrid.setFont(tv.defaultFont);
    layoutGrid.addActionListener(ml);

    JRadioButtonMenuItem layoutGridRandom = new JRadioButtonMenuItem("Grid + Random");
    group.add(layoutGridRandom);
    layoutGridRandom.setFont(tv.defaultFont);
    layoutGridRandom.addActionListener(ml);

    JMenuItem layoutFileLoad = new JMenuItem("File Load");
    layoutFileLoad.addActionListener(ml);
    layoutFileLoad.setFont(tv.defaultFont);

    JMenuItem layoutFileSave = new JMenuItem("File Save");
    layoutFileSave.addActionListener(ml);
    layoutFileSave.setFont(tv.defaultFont);

    layoutMenu.add(layoutRandom);
    layoutMenu.add(layoutGrid);
    layoutMenu.add(layoutGridRandom);
    layoutMenu.add(layoutFileLoad);
    layoutMenu.add(layoutFileSave);

    tv.getMenuBar().addMenu(layoutMenu);
  }

  public void deregister() {
    state.removeSOGenerator(moteSimObjectGenerator);
  }

  public void handleEvent (SimEvent event) {
    if (DEBUG) System.err.println("MOTEPLUGIN: handling "+event);

    if (event instanceof OptionSetEvent) {
      OptionSetEvent ose = (OptionSetEvent)event;
      if (!ose.name.equals("layout")) return;
      if (ose.value.equals("grid")) {
	this.layout = MotePlugin.LAYOUT_GRID;
      } else if (ose.value.equals("random")) {
	this.layout = MotePlugin.LAYOUT_RANDOM;
      } else if (ose.value.equals("gridrandom")) {
	this.layout = MotePlugin.LAYOUT_GRID_RANDOM;
      } else if (ose.value.startsWith("file ")) {
	int oldlayout = this.layout;
	this.layout = MotePlugin.LAYOUT_FILE;
	try {
	  loadLocationFile(new File(ose.value.substring(5)));
	} catch (Exception e) {
	  System.err.println("Can't load location file '"+ose.value.substring(5)+"': "+e);
	  this.layout = oldlayout;
	}
      } else {
	System.err.println("Bad value for layout option: "+ose.value);
      }
      doLayout();
      motePanel.refresh();

    } else if (event instanceof SimObjectEvent) {
      SimObjectEvent soEvent = (SimObjectEvent)event;
      SimObject simObject = soEvent.getSimObject();
      switch (soEvent.getType()) {
	case (SimObjectEvent.OBJECT_ADDED):
	  if (simObject instanceof MoteSimObject) {
	    // XXX MDW: Assume motes aren't added on the fly
	    //if (layout == LAYOUT_GRID || layout == LAYOUT_GRID_RANDOM) 
	    //  doLayout();
	    //motePanel.refresh();
	  }
	  break;
	case (SimObjectEvent.OBJECT_REMOVED):
	  if (simObject instanceof MoteSimObject) {
	    if (layout == LAYOUT_GRID || layout == LAYOUT_GRID_RANDOM) 
	      doLayout();
	    motePanel.refresh();
	  }
	  break;
      }
    }
    else if (event instanceof TossimInitEvent) {
	TossimInitEvent tiEvent = (TossimInitEvent)event;
	int numMotes = tiEvent.get_numMotes();
	for (int i = 0; i < numMotes; i++) {
	    moteSimObjectGenerator.create(i);
	}
	if (layout != LAYOUT_RANDOM) doLayout();
	motePanel.refresh();
    }

    else if (event instanceof TossimEvent) {
      TossimEvent tosEvent = (TossimEvent)event;

      moteSimObjectGenerator.create(tosEvent.getMoteID());

      if (tosEvent instanceof DebugMsgEvent) {
	DebugMsgEvent dmEvent = (DebugMsgEvent)tosEvent;
	boolean attributeModified = false;
	// change the led values accordingly, based on if the debug message has "LEDS:" in it
	String data = dmEvent.getMessage();
	StringTokenizer st = new StringTokenizer(data, " ");
	if (st.countTokens() == 3) {
	  if (st.nextToken().compareTo("LEDS:") == 0) {
	    MoteSimObject mote = state.getMoteSimObject(dmEvent.getMoteID());
	    MoteLedsAttribute ledAttrib = (MoteLedsAttribute) (mote.getAttribute("net.tinyos.sim.MoteLedsAttribute"));
	    String color = st.nextToken();
	    if (color.compareTo("Red") == 0) {
	      if (st.nextToken().compareTo("off.") == 0) {
		ledAttrib.setRedOff();
		attributeModified = true;
	      }
	      else {
		ledAttrib.setRedOn();
		attributeModified = true;
	      }
	    }
	    else if (color.compareTo("Yellow") == 0) {
	      if (st.nextToken().compareTo("off.") == 0) {
		ledAttrib.setYellowOff();
		attributeModified = true;
	      }
	      else {
		ledAttrib.setYellowOn();
		attributeModified = true;
	      }
	    }
	    else if (color.compareTo("Green") == 0) {
	      if (st.nextToken().compareTo("off.") == 0) {
		ledAttrib.setGreenOff();
		attributeModified = true;
	      }
	      else {
		ledAttrib.setGreenOn();
		attributeModified = true;
	      }
	    }
	    if (attributeModified)
	      eventBus.addEvent(new AttributeEvent(ATTRIBUTE_CHANGED, mote, ledAttrib));
	  }
	  motePanel.refresh();
	}

      }
      //System.err.println("MotePlugin handleEvent done.");
    }
  }
  public void draw(Graphics graphics) {
    //System.err.println("MotePlugin draw called.");
    //System.out.println("drawing on graphics " + graphics);
    Iterator it = state.getMoteSimObjects().iterator();
    graphics.setFont(tv.smallFont);

    while (it.hasNext()) {
      MoteSimObject moteSimObject = (MoteSimObject)it.next();

      if (!moteSimObject.isVisible()) {
	continue;
      }

      if (moteSimObject.getPower()) {
	if (moteSimObject.isSelected()) {
  	  graphics.setColor(SELECTED_COLOR);
   	}
    	else {
  	  graphics.setColor(BASIC_COLOR);
   	}
      } else {
	if (moteSimObject.isSelected()) {
  	  graphics.setColor(SELECTED_COLOR_OFF);
   	}
    	else {
  	  graphics.setColor(BASIC_COLOR_OFF);
   	}
      }

      MoteCoordinateAttribute coordinate = moteSimObject.getCoordinate();
      int x = (int)cT.simXToGUIX(coordinate.getX());
      int y = (int)cT.simYToGUIY(coordinate.getY());

      // Drawing Mote itself based on coordinate	    
      int size = moteSimObject.getObjectSize();
      int xl = x-(size/2);
      int yl = y-(size/2);
      graphics.fillOval(xl, yl, size, size);

      // Writing out MoteID
      int id = moteSimObject.getID(); 
      graphics.setColor(Color.black);
      graphics.drawString(Integer.toString(id), xl, yl-1);

      // Drawing Leds
      MoteLedsAttribute leds = (MoteLedsAttribute)moteSimObject.getAttribute("net.tinyos.sim.MoteLedsAttribute");
      if (leds.redLedOn()) {
	graphics.setColor(Color.red);
	graphics.fillRect(xl, y-(size/6), size/3, size/3);
      }
      if (leds.greenLedOn()) {
	graphics.setColor(Color.green);
	graphics.fillRect(xl+(size/3), y-(size/6), size/3, size/3);
      }
      if (leds.yellowLedOn()) {
	graphics.setColor(Color.yellow);
	graphics.fillRect(xl + (2*size/3), y-(size/6), size/3, size/3);
      }
    }
  }

  public String toString() {
    return "MotePlugin";	
  }

  private void loadLocationFile(File locfile) throws IOException {
    if (DEBUG) System.out.println("MOTEPLUGIN: Loading location file "+locfile);
    spatialReader = new SpatialReader(locfile);
    Iterator it = state.getMoteSimObjects().iterator();
    while (it.hasNext()) {
      MoteSimObject mote = (MoteSimObject)it.next();
      SpatialReader.SREntry sEntry = spatialReader.getEntry(mote.getID());
      if (sEntry != null) {
	if (DEBUG) System.out.println("MOTEPLUGIN: Setting "+mote+" location to "+sEntry.getX()+","+sEntry.getY());
	mote.addAttribute(new MoteCoordinateAttribute(sEntry.getX(), sEntry.getY()));
	mote.addCoordinateChangedEvent();
      }
    }
  }

  private void saveLocationFile(File locfile) throws IOException {
    SpatialWriter sWriter = new SpatialWriter(locfile);
    Iterator it = state.getMoteSimObjects().iterator();
    while (it.hasNext()) {
      MoteSimObject mote = (MoteSimObject)it.next();
      MoteCoordinateAttribute coordAttrib = mote.getCoordinate();
      sWriter.writeEntry(mote.getID(), coordAttrib.getX(), coordAttrib.getY());
    }
    sWriter.done();
  }
    
  private void doLayout() {
    if (DEBUG) System.out.println("MOTEPLUGIN: doing layout");
    int nummotes = state.getMoteSimObjects().size();
    if (DEBUG) System.out.println("MOTEPLUGIN: layout: "+nummotes+" motes");
    //if (nummotes == 0) return;

    if (layout == LAYOUT_FILE) {
      if (spatialReader == null) return;
      Iterator it = state.getMoteSimObjects().iterator();
      while (it.hasNext()) {
	MoteSimObject mote = (MoteSimObject)it.next();
	SpatialReader.SREntry sEntry = spatialReader.getEntry(mote.getID());
	if (sEntry != null) {
	  mote.addAttribute(new MoteCoordinateAttribute(sEntry.getX(), sEntry.getY()));
	  mote.addCoordinateChangedEvent();
	}
      }
      return;
    }

    int num_rows = 1, num_columns = 1;
    double xspacing = 0, yspacing = 0;
    int MAX_SPACING = 4*SimConst.MOTE_OBJECT_SIZE;
    double realwidth = cT.getMoteScaleWidth() * 0.9;
    double realheight = cT.getMoteScaleHeight() * 0.9;
    double xbase = 0.0, ybase = 0.0;

    if (layout == LAYOUT_GRID || layout == LAYOUT_GRID_RANDOM) {
      int side = (int)(Math.ceil(Math.sqrt(nummotes * 1.0)));
      num_rows = side;
      num_columns = side;
      xspacing = realwidth / num_columns;
      yspacing = realheight / num_rows;
      if (xspacing > MAX_SPACING) xspacing = MAX_SPACING;
      if (yspacing > MAX_SPACING) yspacing = MAX_SPACING;
      if (nummotes > 2) {
	xbase = 50.0 - xspacing  * ((side-1) / 2.0);
	ybase = 50.0 - yspacing  * ((side-1) / 2.0);
      } else {
	xbase = 50.0 - xspacing  * ((side-1) / 2.0);
	ybase = 50.0;
      }
    }

    Iterator it = state.getMoteSimObjects().iterator();
    while (it.hasNext()) {
      MoteSimObject mote = (MoteSimObject)it.next();
      if (layout == LAYOUT_RANDOM) {
	  addRandomMoteCoordAttribute(mote);
      } else if (layout == LAYOUT_GRID || layout == LAYOUT_GRID_RANDOM) {
	int rownum = mote.getID() / num_columns;
	int colnum = mote.getID() % num_columns;
	double x = (xspacing * colnum) + xbase;
	double y = (yspacing * rownum) + ybase;
	if (layout == LAYOUT_GRID_RANDOM) {
	  x += ((rand.nextDouble() * GRID_RANDOM_DEVIATION*2.0) - GRID_RANDOM_DEVIATION);
	  y += ((rand.nextDouble() * GRID_RANDOM_DEVIATION*2.0) - GRID_RANDOM_DEVIATION);
	}
	mote.addAttribute(new MoteCoordinateAttribute(x, y));
      }
      mote.addCoordinateChangedEvent();
    }
  }
    
  public void addRandomMoteCoordAttribute(MoteSimObject mote) {
      double x = Math.random()*(cT.getMoteScaleWidth() * 0.8);
      double y = Math.random()*(cT.getMoteScaleHeight() * 0.8);
      x += (cT.getMoteScaleWidth() * 0.1);
      y += (cT.getMoteScaleHeight() * 0.1);
      mote.addAttribute(new MoteCoordinateAttribute(x, y));
  }

  public void setLayout(int thelayout) {
    if (thelayout != LAYOUT_GRID &&
	thelayout != LAYOUT_RANDOM &&
	thelayout != LAYOUT_GRID_RANDOM &&
	thelayout != LAYOUT_FILE) {
      throw new IllegalArgumentException("Invalid setting for layout: "+layout);
    }
    this.layout = thelayout;
    doLayout();
  }
    
  class bPowerListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      Iterator it = state.getSelectedMoteSimObjects().iterator();
      try {
	while (it.hasNext()) {
	  MoteSimObject mote = (MoteSimObject)it.next();
	  // Toggle power status for each mote
	  if (mote.getPower()) {
	    simComm.sendCommand(new TurnOffMoteCommand((short)mote.getID(), 0L));
	    mote.setPower(false);
  	  } else {
  	    simComm.sendCommand(new TurnOnMoteCommand((short)mote.getID(), 0L));
  	    mote.setPower(true);
  	  }
  	  motePanel.refresh();
   	}
      } catch (IOException ioe) {
	System.err.println("Cannot send command: "+ioe);
      }
    }
  }

  class menuListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      if (e.getActionCommand().equals("Grid")) {
	layout = LAYOUT_GRID;
	doLayout();
      } else if (e.getActionCommand().equals("Random")) {
	layout = LAYOUT_RANDOM;
	doLayout();
      } else if (e.getActionCommand().equals("Grid + Random")) {
	layout = LAYOUT_GRID_RANDOM;
	doLayout();
      } else if (e.getActionCommand().equals("File Load")) {
	int oldlayout = layout;
	layout = LAYOUT_FILE;
	JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
	fc.addChoosableFileFilter(new MotePositionsFileFilter());
	int returnVal = fc.showOpenDialog(motePanel);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File file = fc.getSelectedFile();
	    try {	    
	      loadLocationFile(file);
	    } catch (IOException exception) {
      	      tv.setStatus("Error reading location file "+file.getName());
	      layout = oldlayout;
	    }
	} else {
	  layout = oldlayout;
	}
      } else if (e.getActionCommand().equals("File Save")) {
	  JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
	  fc.addChoosableFileFilter(new MotePositionsFileFilter());
	  int returnVal = fc.showSaveDialog(motePanel);
	  if (returnVal == JFileChooser.APPROVE_OPTION) {
	      File file = fc.getSelectedFile();
	      try {
		saveLocationFile(file);
	      } catch (IOException ioException) {
      		tv.setStatus("Error writing spatial file "+file.getName());
		System.out.println(ioException);
		ioException.printStackTrace();
	      }
	  }
      }
      motePanel.refresh();
    }

    protected class MotePositionsFileFilter extends javax.swing.filechooser.FileFilter {
      public MotePositionsFileFilter() {}

      public boolean accept(File f) {
	if (f.isDirectory()) {
	  return true;
	}
	String name = f.getName();
	int index = name.lastIndexOf('.');

	if (index < 0) {return false;}

	String extension = name.substring(index);
	if (extension != null) {
	  if (extension.equals(".mps")) {
	    return true;
	  } else {
	    return false;
	  }
	}
	return false;
      }
      public String getDescription() {
	return "*.mps";
      }
    }
  }   
}

